@@ -18,6 +18,8 @@
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <libart_lgpl/libart.h>
 
+#include "egg-screen-help.h"
+
 #define FISH_PREFS_NAME		"name"
 #define FISH_PREFS_IMAGE	"image"
 #define FISH_PREFS_COMMAND	"command"
@@ -28,7 +30,10 @@
 /* macro to always pass a non-null string */
 #define sure_string(__x) ((__x)!=NULL?(__x):"")
 
-int fools_day=1, fools_month=3;
+static int fools_day        = 1;
+static int fools_month      = 3;
+static int fools_hour_start = 0;
+static int fools_hour_end   = 12; /* Apparently jokes should stop at midday */
 
 typedef struct {
 	GtkWidget         *applet;
@@ -53,6 +58,13 @@ typedef struct {
 	PanelAppletOrient  orient;
 } Fish;
 
+static inline GdkWindow *
+fishy_root_window (Fish *fish)
+{
+	return gdk_screen_get_root_window (
+			gtk_widget_get_screen (fish->applet));
+}
+
 /*
  * set_access_name_desc
  * Description : Set Accessible Name and Description.
@@ -282,7 +294,7 @@ load_image_file (Fish *fish)
 			fish->w = frames * fish->size;
 			fish->h = fish->size;
 		}
- 		fish->pix = gdk_pixmap_new (GDK_ROOT_PARENT (),
+ 		fish->pix = gdk_pixmap_new (fishy_root_window (fish),
 					    fish->w, fish->h, -1);
 		return;
 	}
@@ -348,7 +360,7 @@ load_image_file (Fish *fish)
 	fish->w = w;
 	fish->h = h;
 
-	fish->pix = gdk_pixmap_new (GDK_ROOT_PARENT (),
+	fish->pix = gdk_pixmap_new (fishy_root_window (fish),
 				    fish->w,fish->h,-1);
 
 	gc = gdk_gc_new (fish->pix);
@@ -443,19 +455,24 @@ fish_timeout(gpointer data)
 	time (&ourtime);
 	tm = localtime (&ourtime);
 
-	if(fish->april_fools) {
-		if(tm->tm_mon != fools_month || tm->tm_mday != fools_day) {
+	if (fish->april_fools) {
+		if (tm->tm_mon  != fools_month ||
+		    tm->tm_mday != fools_day   ||
+		    tm->tm_hour >= fools_hour_end) {
 			fish->april_fools = FALSE;
-			load_image_file(fish);
-			setup_size(fish);
-			fish_timeout(fish);
+			load_image_file (fish);
+			setup_size (fish);
+			fish_timeout (fish);
 		}
 	} else {
-		if(tm->tm_mon == fools_month && tm->tm_mday == fools_day) {
+		if (tm->tm_mon  == fools_month      &&
+		    tm->tm_mday == fools_day        &&
+		    tm->tm_hour >= fools_hour_start &&
+		    tm->tm_hour <= fools_hour_end) {
 			fish->april_fools = TRUE;
-			load_image_file(fish);
-			setup_size(fish);
-			fish_timeout(fish);
+			load_image_file (fish);
+			setup_size (fish);
+			fish_timeout (fish);
 		}
 	}
 
@@ -576,17 +593,28 @@ fish_properties_apply (GtkDialog *pb, Fish *fish)
 		    !strcmp  (text, "uptime")  ||
 		    !strncmp (text, "tail ", 5)) {
 			GtkWidget *w;
+			const gchar *warning_format = _("Warning:  The command "
+					"appears to be something actually useful.\n"
+					"Since this is a useless applet, you "
+					"may not want to do this.\n"
+					"We strongly advise you against "
+					"usage of %s for anything\n"
+					"which would make the applet "
+					"\"practical\" or useful.");
+			gchar *name, *message;
+
+			name = panel_applet_gconf_get_string (PANEL_APPLET (fish->applet),
+							      FISH_PREFS_NAME,
+							      NULL);
+			message = g_strdup_printf (warning_format, name);
 			w = gtk_message_dialog_new (NULL, 0, GTK_MESSAGE_WARNING,
-						    GTK_BUTTONS_OK,
-						    _("Warning:  The command appears to be "
-						      "something actually useful.\n"
-						      "Since this is a useless applet, you "
-						      "may not want to do this.\n"
-						      "We strongly advise you against "
-						      "usage of wanda for anything\n"
-						      "which would make the applet "
-						      "\"practical\" or useful."));
+						    GTK_BUTTONS_OK, message);
+			g_free (message);
+			g_free (name);
+
 			gtk_window_set_wmclass (GTK_WINDOW (w), "fish_useful_warning", "Fish");
+			gtk_window_set_screen (GTK_WINDOW (w),
+					       gtk_widget_get_screen (fish->applet));
 
 			gtk_widget_show_all (w);
 
@@ -617,12 +645,14 @@ fish_properties_apply (GtkDialog *pb, Fish *fish)
 }
 
 static void
-phelp (void)
+phelp (Fish *fish)
 {
 	GError *error = NULL;
 
-	gnome_help_display_desktop (
-		NULL, "fish-applet-2", "fish-applet-2", NULL, &error);
+	egg_help_display_desktop_on_screen (
+		NULL, "fish-applet-2", "fish-applet-2", NULL,
+		gtk_widget_get_screen (fish->applet),
+		&error);
 	if (error) {
 		g_warning ("help error: %s\n", error->message);
 		g_error_free (error);
@@ -651,7 +681,7 @@ response_cb (GtkDialog *dialog, gint id, gpointer data)
 			fish_properties_apply (dialog, fish);
 			break;
 		case GTK_RESPONSE_HELP:
-			phelp ();
+			phelp (fish);
 			break;
 		case GTK_RESPONSE_CLOSE:
 			gtk_widget_destroy (GTK_WIDGET(dialog));
@@ -681,7 +711,9 @@ display_properties_dialog (BonoboUIComponent *uic,
 	gboolean       rotate;
 	GtkWidget     *apply_button;
 
-	if (fish->pb != NULL) {
+	if (fish->pb) {
+		gtk_window_set_screen (GTK_WINDOW (fish->pb),
+				       gtk_widget_get_screen (fish->applet));
 		gtk_window_present (GTK_WINDOW (fish->pb));
 		return;
 	}
@@ -705,6 +737,8 @@ display_properties_dialog (BonoboUIComponent *uic,
 				GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE, 
 				GTK_STOCK_OK, GTK_RESPONSE_OK, NULL);
 	gtk_window_set_wmclass (GTK_WINDOW (fish->pb), "fish", "Fish");
+	gtk_window_set_screen (GTK_WINDOW (fish->pb),
+			       gtk_widget_get_screen (fish->applet));
 
 	gtk_dialog_set_default_response (GTK_DIALOG (fish->pb), GTK_RESPONSE_OK);
 
@@ -835,7 +869,8 @@ display_properties_dialog (BonoboUIComponent *uic,
 }
 
 static void
-something_fishy_going_on (const char *message)
+something_fishy_going_on (Fish       *fish,
+			  const char *message)
 {
 	GtkWidget *dialog;
 
@@ -850,6 +885,8 @@ something_fishy_going_on (const char *message)
 			  NULL);
 
 	gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
+	gtk_window_set_screen (GTK_WINDOW (dialog),
+			       gtk_widget_get_screen (fish->applet));
 	gtk_widget_show (dialog);
 }
 
@@ -860,7 +897,7 @@ fish_locate_fortune_command (Fish *fish)
 
 	retval = panel_applet_gconf_get_string (PANEL_APPLET (fish->applet), FISH_PREFS_COMMAND, NULL);
 	if (!retval) {
-		something_fishy_going_on (_("Unable to get the name of the command to execute"));
+		something_fishy_going_on (fish, _("Unable to get the name of the command to execute"));
 		return NULL;
 	}
 
@@ -876,7 +913,7 @@ fish_locate_fortune_command (Fish *fish)
 		retval = g_strdup ("/usr/games/fortune");
 
 	if (!retval)
-		something_fishy_going_on (_("Unable to locate the command to execute"));
+		something_fishy_going_on (fish, _("Unable to locate the command to execute"));
 
 	return retval;
 }
@@ -951,6 +988,8 @@ update_fortune_dialog (Fish *fish)
 		g_signal_connect (G_OBJECT (fish->fortune_dialog), "response",
 				  G_CALLBACK (response), NULL);
 		gtk_window_set_wmclass (GTK_WINDOW (fish->fortune_dialog), "fish", "Fish");
+		gtk_window_set_screen (GTK_WINDOW (fish->fortune_dialog),
+				       gtk_widget_get_screen (fish->applet));
 		gnome_window_icon_set_from_file (GTK_WINDOW (fish->fortune_dialog),
 						 GNOME_ICONDIR"/gnome-fish.png");
 
@@ -992,6 +1031,8 @@ update_fortune_dialog (Fish *fish)
 		gtk_widget_show_all (fish->fortune_dialog);
 	} else {
 		set_access_name_desc (NULL, fish);
+		gtk_window_set_screen (GTK_WINDOW (fish->fortune_dialog),
+				       gtk_widget_get_screen (fish->applet));
 		gtk_window_present (GTK_WINDOW (fish->fortune_dialog));
 	}
 
@@ -1003,7 +1044,7 @@ update_fortune_dialog (Fish *fish)
 
 		message = g_strdup_printf (_("Unable to execute '%s'\n\nDetails : %s"),
 					   fortune_command, error->message);
-		something_fishy_going_on (message);
+		something_fishy_going_on (fish, message);
 		g_free (message);
 		g_error_free (error);
 	}
@@ -1039,6 +1080,8 @@ change_water (Fish *fish)
 				    _("The water needs changing!\n"
 				    "(Look at today's date)"));
 	gtk_window_set_wmclass (GTK_WINDOW (w), "fish", "Fish");
+	gtk_window_set_screen (GTK_WINDOW (w),
+			       gtk_widget_get_screen (fish->applet));
 
 	gtk_widget_show_all (w);
 
@@ -1184,8 +1227,10 @@ display_help_dialog (BonoboUIComponent *uic,
 {
 	GError *error = NULL;
 
-	gnome_help_display_desktop (
-		NULL, "fish-applet-2", "fish-applet-2", NULL, &error);
+	egg_help_display_desktop_on_screen (
+		NULL, "fish-applet-2", "fish-applet-2", NULL,
+		gtk_widget_get_screen (fish->applet),
+		&error);
 	if (error) {
 		g_warning ("help error: %s\n", error->message);
 		g_error_free (error);
@@ -1201,13 +1246,23 @@ display_about_dialog (BonoboUIComponent *uic,
 		      const gchar       *verbname)
 {
 	const gchar *author_format = _("%s the Fish");
+	const gchar *about_format = _("%s has no use what-so-ever. "
+				   "It only takes up disk space and "
+				   "compilation time, and if loaded it also "
+				   "takes up precious panel space and memory. "
+				   "If anyone is found using it, he "
+				   "should be promptly sent for a psychiatric "
+				   "evaluation.");
 	gchar       *authors [3];
 	GdkPixbuf   *pixbuf;
 	GError      *error = NULL;
 	gchar       *file;
 	gchar       *name;
+	gchar       *descr;
 
-	if (fish->aboutbox != NULL) {
+	if (fish->aboutbox) {
+		gtk_window_set_screen (GTK_WINDOW (fish->aboutbox),
+				       gtk_widget_get_screen (fish->applet));
 		gtk_window_present (GTK_WINDOW (fish->aboutbox));
 		return;
 	}
@@ -1218,8 +1273,6 @@ display_about_dialog (BonoboUIComponent *uic,
 	authors[1] = _("(with minor help from George)");
 	authors[2] = NULL;
 
-	g_free (name);
-
 	file = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_PIXMAP, "gnome-fish.png", FALSE, NULL);
 
 	pixbuf = gdk_pixbuf_new_from_file (file, &error);
@@ -1231,27 +1284,28 @@ display_about_dialog (BonoboUIComponent *uic,
 
 		g_error_free (error);
 	}
+
+	descr = g_strdup_printf (about_format, name);
 		
 	fish->aboutbox =
 		gnome_about_new (_("Fish"),
 				 "3.4.7.4ac19",
 				 "(C) 1998 the Free Software Foundation",
-				 _("This applet has no use what-so-ever. "
-				   "It only takes up disk space and "
-				   "compilation time, and if loaded it also "
-				   "takes up precious panel space and memory. "
-				   "If anyone is found using this applet, he "
-				   "should be promptly sent for a psychiatric "
-				   "evaluation."),
+				 descr,
 				 (const char **)authors,
 				 NULL,
 				 NULL,
 				 pixbuf);
 
+	g_free (descr);
+	g_free (name);
+
 	if (pixbuf)
 		gdk_pixbuf_unref (pixbuf);
 
 	gtk_window_set_wmclass (GTK_WINDOW (fish->aboutbox), "fish", "Fish");
+	gtk_window_set_screen (GTK_WINDOW (fish->aboutbox),
+			       gtk_widget_get_screen (fish->applet));
 	gnome_window_icon_set_from_file (GTK_WINDOW (fish->aboutbox),
 					 GNOME_ICONDIR"/gnome-fish.png");
 	gtk_signal_connect (GTK_OBJECT (fish->aboutbox), "destroy",
