@@ -37,10 +37,10 @@
 #include "foobar-widget.h"
 #include "gnome-run.h"
 #include "launcher.h"
-#include "logout.h"
 #include "menu-fentry.h"
 #include "menu-util.h"
 #include "menu.h"
+#include "multiscreen-stuff.h"
 #include "panel-util.h"
 #include "panel-config.h"
 #include "panel-config-global.h"
@@ -48,6 +48,7 @@
 #include "session.h"
 #include "panel-applet-frame.h"
 #include "global-keys.h"
+#include "panel-action-button.h"
 
 #define PANEL_EVENT_MASK (GDK_BUTTON_PRESS_MASK |		\
 			   GDK_BUTTON_RELEASE_MASK |		\
@@ -158,19 +159,9 @@ static GConfEnumStringPair panel_speed_type_enum_map [] = {
 	{ PANEL_SPEED_FAST,   "panel-speed-fast" },
 };
 
-static void
-change_window_cursor(GdkWindow *window, GdkCursorType cursor_type)
-{
-	GdkCursor *cursor = gdk_cursor_new (cursor_type);
-	gdk_window_set_cursor (window, cursor);
-	gdk_cursor_unref (cursor);
-}
-
 static void
 panel_realize (GtkWidget *widget, gpointer data)
 {
-	change_window_cursor (widget->window, GDK_LEFT_PTR);
-	
 	if (BASEP_IS_WIDGET (widget))
 		basep_widget_enable_buttons(BASEP_WIDGET(widget), TRUE);
 	else if (FOOBAR_IS_WIDGET (widget))
@@ -497,9 +488,16 @@ menu_deactivate(GtkWidget *w, PanelData *pd)
 static void
 move_panel_to_cursor (GtkWidget *panel)
 {
+	GdkScreen *screen;
+	GdkWindow *root_window;
 	int        x, y;
 
-	gdk_window_get_pointer (gdk_get_default_root_window (), &x, &y, NULL);
+	g_return_if_fail (GTK_IS_WINDOW (panel));
+
+	screen = gtk_window_get_screen (GTK_WINDOW (panel));
+	root_window = gdk_screen_get_root_window (screen);
+
+	gdk_window_get_pointer (root_window, &x, &y, NULL);
 
 	if (BASEP_IS_WIDGET (panel))
 		basep_widget_set_pos (BASEP_WIDGET (panel), x, y);
@@ -664,6 +662,7 @@ static gboolean
 panel_do_popup_menu (PanelWidget *panel,
 		     BasePWidget *basep,
 		     GtkWidget   *widget,
+		     int          screen,
 		     guint        button,
 		     guint32      activate_time)
 {
@@ -676,6 +675,9 @@ panel_do_popup_menu (PanelWidget *panel,
 			basep_widget_autohide (basep);
 		}
 
+		gtk_menu_set_screen (GTK_MENU (menu),
+				     panel_screen_from_number (screen));
+
 		gtk_menu_popup (GTK_MENU (menu),
                                 NULL,
                                 NULL,
@@ -694,13 +696,17 @@ panel_popup_menu (GtkWidget *widget, gpointer data)
 {
  	PanelWidget *panel = NULL;
 	BasePWidget *basep = NULL;
+	int          screen = 0;
 
 	panel = PANEL_WIDGET (widget);
-	if (BASEP_IS_WIDGET (data))
+	if (BASEP_IS_WIDGET (data)) {
 		basep = BASEP_WIDGET (data);
+		screen = basep->screen;
+	} else if (FOOBAR_IS_WIDGET (data))
+		screen = FOOBAR_WIDGET (data)->screen;
 
 	return panel_do_popup_menu (
-			panel, basep, widget, 3, GDK_CURRENT_TIME);
+			panel, basep, widget, screen, 3, GDK_CURRENT_TIME);
 }
 
 static gboolean
@@ -738,12 +744,15 @@ panel_event(GtkWidget *widget, GdkEvent *event)
 	GdkEventButton *bevent;
 	GdkEventKey *kevent;
 	int x, y;
+	int screen = 0;
 
 	if (BASEP_IS_WIDGET (widget)) {
 		basep = BASEP_WIDGET (widget);
 		panel = PANEL_WIDGET (basep->panel);
+		screen = basep->screen;
 	} else if (FOOBAR_IS_WIDGET (widget)) {
 		panel = PANEL_WIDGET (FOOBAR_WIDGET (widget)->panel);
+		screen= FOOBAR_WIDGET (widget)->screen;
 	}
 
 	switch (event->type) {
@@ -763,7 +772,7 @@ panel_event(GtkWidget *widget, GdkEvent *event)
 			else
 				pd->insertion_pos = x;
 
-			if (panel_do_popup_menu (panel, basep, widget,
+			if (panel_do_popup_menu (panel, basep, widget, screen,
 						 bevent->button, bevent->time))
 				return TRUE;
 			break;
@@ -1168,27 +1177,16 @@ drop_internal_applet (PanelWidget *panel, int pos, const char *applet_type,
 		else
 			drop_menu (panel, pos, menu);
 
-	} else if (strcmp(applet_type,"DRAWER:NEW")==0) {
-		load_drawer_applet(NULL, NULL, NULL, panel, pos, TRUE, NULL);
+	} else if (!strcmp (applet_type, "DRAWER:NEW"))
+		load_drawer_applet (NULL, NULL, NULL, panel, pos, TRUE, NULL);
 
-	} else if (strcmp (applet_type, "LOGOUT:NEW") == 0) {
-		load_logout_applet (panel, pos, TRUE, NULL);
+	else if (!strncmp (applet_type, "ACTION:", strlen ("ACTION:")))
+		remove_applet = panel_action_button_load_from_drag (
+					applet_type, panel, pos,
+					TRUE, NULL, &applet_num);
 
-	} else if (sscanf (applet_type, "LOGOUT:%d", &applet_num) == 1) {
-		load_logout_applet (panel, pos, TRUE, NULL);
-		remove_applet = TRUE;
-
-	} else if (strcmp (applet_type, "LOCK:NEW") == 0) {
-		load_lock_applet (panel, pos, TRUE, NULL);
-
-	} else if (sscanf (applet_type, "LOCK:%d", &applet_num) == 1) {
-		load_lock_applet (panel, pos, TRUE, NULL);
-		remove_applet = TRUE;
-
-	} else if(strcmp(applet_type,"LAUNCHER:ASK")==0) {
-		ask_about_launcher(NULL, panel, pos, TRUE);
-
-	}
+	else if (!strcmp(applet_type,"LAUNCHER:ASK"))
+		ask_about_launcher (NULL, panel, pos, TRUE);
 
 	if (remove_applet &&
 	    action == GDK_ACTION_MOVE) {
@@ -1200,18 +1198,21 @@ drop_internal_applet (PanelWidget *panel, int pos, const char *applet_type,
 }
 
 static void
-drop_color(PanelWidget *panel, int pos, guint16 *dropped)
+drop_color (PanelWidget *panel,
+	    int          pos,
+	    guint16     *dropped)
 {
-	GdkColor c;
+	PanelColor c;
 
-	if(!dropped) return;
+	if (!dropped)
+		return;
 
-	c.red = dropped[0];
-	c.green = dropped[1];
-	c.blue = dropped[2];
-	c.pixel = 0;
+	c.red   = dropped [0];
+	c.green = dropped [1];
+	c.blue  = dropped [2];
+	c.alpha = 65535;
 
-	panel_widget_set_back_color(panel, &c);
+	panel_widget_set_back_color (panel, &c);
 }
 
 static GtkTargetList *
@@ -1735,6 +1736,29 @@ panel_data_by_id (const char *id)
 	return NULL;
 }
 
+static void
+panel_ensure_panel_per_screen (int screen)
+{
+	GtkWidget *panel;
+	GSList    *l;
+
+	for (l = panel_list; l; l = l->next) {
+		PanelData *pdata = l->data;
+
+		if (BASEP_IS_WIDGET (pdata->panel) &&
+		    BASEP_WIDGET (pdata->panel)->screen == screen)
+			return;
+
+		else if (FOOBAR_IS_WIDGET (pdata->panel) &&
+			 FOOBAR_WIDGET (pdata->panel)->screen == screen)
+			return;
+	}
+
+	panel = foobar_widget_new (NULL, screen, 0);
+	panel_save_to_gconf (panel_setup (panel));
+	gtk_widget_show (panel);
+}
+
 void
 panel_load_global_config (void)
 {
@@ -1935,6 +1959,7 @@ panel_load_panels_from_gconf (void)
 	GSList     *l;
 	const char *profile;
 	const char *key;
+	int         i;
 
 	profile = panel_gconf_get_profile ();
 	
@@ -1945,27 +1970,41 @@ panel_load_panels_from_gconf (void)
 
 	for (l = panel_ids; l; l = l->next) {
 		GtkWidget     *panel = NULL;
-		PanelType      type;
-		PanelBackType  back_type;
+		PanelType      type = EDGE_PANEL;
+		PanelBackType  back_type = PANEL_BACK_NONE;
 		BasePState     state;
 		BasePMode      mode;
-		GdkColor       back_color = {0,0,0,1};
+		PanelColor     back_color = {0, 0, 0, 0xFFFF};
+		GdkColor       gdkcolor = {0, 0, 0, 1};
 		gboolean       fit_pixmap_bg;
 		gboolean       stretch_pixmap_bg;
 		gboolean       rotate_pixmap_bg;
 		gboolean       hidebuttons_enabled;
 		gboolean       hidebutton_pixmaps_enabled;
 		int            screen;
-		int            size;
+		int            monitor;
+		int            size = PANEL_SIZE_SMALL;
 		char          *panel_id;
 		char          *back_pixmap;
 		char          *tmp_str;
 
 		panel_id = l->data;
 
-#ifdef PANEL_SESSION_DEBUG
-		printf ("Loading panel id %s\n", panel_id);
-#endif
+		screen  = panel_get_int (profile, panel_id, "screen", -1);
+		monitor = panel_get_int (profile, panel_id, "monitor", -1);
+		if (screen == -1 || monitor == -1) {
+			/* Backwards compat:
+			 *   In 2.0.0, we only had Xinerama support and the
+			 *   monitor number was saved as "screen_id".
+			 */
+			screen = 0;
+			monitor = panel_get_int (profile, panel_id, "screen_id", 0);
+		}
+
+		if (screen >= multiscreen_screens () ||
+		    monitor >= multiscreen_monitors (screen))
+			continue;
+
 		back_pixmap = panel_get_string (profile, panel_id,
 						"panel_background_pixmap", NULL);
 		if (string_empty (back_pixmap)) {
@@ -1974,12 +2013,19 @@ panel_load_panels_from_gconf (void)
 		}
 
 		tmp_str = panel_get_string (profile, panel_id, "panel_background_color", NULL);
-		if (!string_empty (tmp_str))
-			gdk_color_parse (tmp_str, &back_color);
+		if (!tmp_str || !tmp_str [0]) {
+			gdk_color_parse (tmp_str, &gdkcolor);
+			back_color.red   = gdkcolor.red;
+			back_color.green = gdkcolor.green;
+			back_color.blue  = gdkcolor.blue;
+		}
 		g_free (tmp_str);
 
+		back_color.alpha = panel_get_int (
+					profile, panel_id, "panel_background_color_alpha", 0xFFFF);
+
 		tmp_str = panel_get_string (
-				profile, panel_id, "panel_background_type", "no_background");
+				profile, panel_id, "panel_background_type", "no-background");
 		gconf_string_to_enum (background_type_enum_map, tmp_str, (int *) &back_type);
 		g_free (tmp_str);
 		
@@ -2004,15 +2050,14 @@ panel_load_panels_from_gconf (void)
 
 		state   = panel_get_int (profile, panel_id, "panel_hide_state", 0);
 		mode    = panel_get_int (profile, panel_id, "panel_hide_mode", 0);
-		screen  = panel_get_int (profile, panel_id, "screen_id", 0);
 
 		tmp_str = panel_get_string (profile, panel_id, "panel_type", "edge-panel");
-		gconf_string_to_enum (panel_type_type_enum_map, tmp_str, (gint *) &type);
+		gconf_string_to_enum (panel_type_type_enum_map, tmp_str, (int *) &type);
 		g_free (tmp_str);
 
 		switch (type) {
 		case EDGE_PANEL: {
-			BorderEdge edge;
+			BorderEdge edge = BORDER_BOTTOM;
 
 			tmp_str = panel_get_string (profile, panel_id,
 						    "screen_edge", "panel-edge-bottom");
@@ -2022,6 +2067,7 @@ panel_load_panels_from_gconf (void)
 			
 			panel = edge_widget_new (panel_id,
 						 screen,
+						 monitor,
 						 edge, 
 						 mode, state,
 						 size,
@@ -2035,8 +2081,8 @@ panel_load_panels_from_gconf (void)
 			}
 			break;
 		case ALIGNED_PANEL: {
-			AlignedAlignment align;
-			BorderEdge       edge;
+			AlignedAlignment align = ALIGNED_LEFT;
+			BorderEdge       edge = BORDER_BOTTOM;
 
 			tmp_str = panel_get_string (profile, panel_id,
 						    "screen_edge", "panel-edge-bottom");
@@ -2052,6 +2098,7 @@ panel_load_panels_from_gconf (void)
 
 			panel = aligned_widget_new (panel_id,
 						    screen,
+						    monitor,
 						    align,
 						    edge,
 						    mode,
@@ -2067,8 +2114,8 @@ panel_load_panels_from_gconf (void)
 			}
 			break;
 		case SLIDING_PANEL: {
-			SlidingAnchor anchor;
-			BorderEdge    edge;
+			SlidingAnchor anchor = SLIDING_ANCHOR_LEFT;
+			BorderEdge    edge = BORDER_BOTTOM;
 			gint16        offset;
 
 			tmp_str = panel_get_string (profile, panel_id,
@@ -2087,6 +2134,7 @@ panel_load_panels_from_gconf (void)
 	
 			panel = sliding_widget_new (panel_id,
 						    screen,
+						    monitor,
 						    anchor,
 						    offset,
 						    edge,
@@ -2103,7 +2151,7 @@ panel_load_panels_from_gconf (void)
 			}
 			break;
 		case DRAWER_PANEL: {
-			int orient;
+			int orient = PANEL_ORIENT_UP;
 
 			tmp_str = panel_get_string (profile, panel_id,
 						    "panel_orient", "panel-orient-up");
@@ -2112,6 +2160,8 @@ panel_load_panels_from_gconf (void)
 			g_free (tmp_str);
 
 			panel = drawer_widget_new (panel_id,
+						   screen,
+						   monitor,
 						   (PanelOrient) orient,
 						   BASEP_EXPLICIT_HIDE, 
 						   state,
@@ -2126,7 +2176,7 @@ panel_load_panels_from_gconf (void)
 			}
 			break;
 		case FLOATING_PANEL: {
-			GtkOrientation orient;
+			GtkOrientation orient = GTK_ORIENTATION_HORIZONTAL;
 			int            x, y;
 
 			tmp_str = panel_get_string (profile, panel_id,
@@ -2140,6 +2190,7 @@ panel_load_panels_from_gconf (void)
 			
 			panel = floating_widget_new (panel_id,
 						     screen,
+						     monitor,
 						     x,
 						     y,
 						     orient,
@@ -2156,7 +2207,7 @@ panel_load_panels_from_gconf (void)
 			}
 			break;
 		case FOOBAR_PANEL:
-			panel = foobar_widget_new (panel_id, screen);
+			panel = foobar_widget_new (panel_id, screen, monitor);
 			break;
 		default:
 			g_warning ("Unkown panel type: %d; ignoring.", type);
@@ -2176,15 +2227,19 @@ panel_load_panels_from_gconf (void)
 		GtkWidget *panel;
 
 		panel_error_dialog (
+			gdk_screen_get_default (),
 			"no_panels_found",
 			_("No panels were found in your configuration.  "
 			  "I will create a menu panel for you"));
 
-		panel = foobar_widget_new (NULL, 0);
+		panel = foobar_widget_new (NULL, 0, 0);
 		panel_save_to_gconf (panel_setup (panel));
 		gtk_widget_show (panel);
 	}
 
+	for (i = 0; i < multiscreen_screens (); i++)
+		panel_ensure_panel_per_screen (i);
+
 	panel_g_slist_deep_free (panel_ids);
 }
 
@@ -2200,6 +2255,7 @@ panel_save_to_gconf (PanelData *pd)
 	const char  *key;
 	char	    *color;
 	int          screen = 0;
+	int          monitor = 0;
 
 	g_return_if_fail (pd != NULL);
 	
@@ -2207,10 +2263,12 @@ panel_save_to_gconf (PanelData *pd)
 		basep   = BASEP_WIDGET (pd->panel);
 		panel   = PANEL_WIDGET (basep->panel);
 		screen  = basep->screen;
+		monitor = basep->monitor;
 
 	} else if (FOOBAR_IS_WIDGET (pd->panel)) {
 		panel   = PANEL_WIDGET (FOOBAR_WIDGET (pd->panel)->panel);
 		screen  = FOOBAR_WIDGET (pd->panel)->screen;
+		monitor = FOOBAR_WIDGET (pd->panel)->monitor;
 	}
 
 	profile = panel_gconf_get_profile ();
@@ -2238,7 +2296,8 @@ panel_save_to_gconf (PanelData *pd)
 	panel_set_string (profile, panel->unique_id, "panel_type", 
 			  gconf_enum_to_string (panel_type_type_enum_map, pd->type));
 
-	panel_set_int (profile, panel->unique_id, "screen_id", screen);
+	panel_set_int (profile, panel->unique_id, "screen", screen);
+	panel_set_int (profile, panel->unique_id, "monitor", monitor);
 
 	if (basep) {
 		panel_set_bool (profile, panel->unique_id,
@@ -2284,6 +2343,10 @@ panel_save_to_gconf (PanelData *pd)
 			  "panel_background_color", color);
 	g_free (color);
 
+	panel_set_int (profile, panel->unique_id,
+		       "panel_background_color_alpha",
+		       panel->back_color.alpha);
+
 	panel_set_string (profile, panel->unique_id,
 			  "panel_background_type", 
 			  gconf_enum_to_string (background_type_enum_map, panel->back_type));
@@ -2386,20 +2449,67 @@ panel_register_window_icon (void)
 	}
 }
 
+GdkScreen *
+panel_screen_from_toplevel (GtkWidget *panel)
+{
+	GdkScreen  *retval = NULL;
+	GdkDisplay *display;
+
+	display = gdk_display_get_default ();
+
+	if (BASEP_IS_WIDGET (panel))
+	        retval = gdk_display_get_screen (
+	                        display, BASEP_WIDGET (panel)->screen);
+
+	else if (FOOBAR_IS_WIDGET (panel))
+	        retval = gdk_display_get_screen (
+	                        display, FOOBAR_WIDGET (panel)->screen);
+
+	return retval;
+}
+
 int
 panel_monitor_from_toplevel (GtkWidget *panel)
 {
 	int retval = -1;
 
 	if (BASEP_IS_WIDGET (panel))
-	        retval = BASEP_WIDGET (panel)->screen;
+	        retval = BASEP_WIDGET (panel)->monitor;
 
 	else if (FOOBAR_IS_WIDGET (panel))
-	        retval = FOOBAR_WIDGET (panel)->screen;
+	        retval = FOOBAR_WIDGET (panel)->monitor;
 
 	return retval;
 }
 
+int
+panel_screen_from_panel_widget (PanelWidget *panel)
+{
+	g_return_val_if_fail (PANEL_IS_WIDGET (panel), 0);
+	g_return_val_if_fail (panel->panel_parent != NULL, 0);
+	g_return_val_if_fail (BASEP_IS_WIDGET (panel->panel_parent) ||
+			      FOOBAR_IS_WIDGET (panel->panel_parent), 0);
+
+	if (BASEP_IS_WIDGET (panel->panel_parent))
+		return BASEP_WIDGET (panel->panel_parent)->screen;
+	else
+		return FOOBAR_WIDGET (panel->panel_parent)->screen;
+}
+
+int
+panel_monitor_from_panel_widget (PanelWidget *panel)
+{
+	g_return_val_if_fail (PANEL_IS_WIDGET (panel), 0);
+	g_return_val_if_fail (panel->panel_parent != NULL, 0);
+	g_return_val_if_fail (BASEP_IS_WIDGET (panel->panel_parent) ||
+			      FOOBAR_IS_WIDGET (panel->panel_parent), 0);
+
+	if (BASEP_IS_WIDGET (panel->panel_parent))
+		return BASEP_WIDGET (panel->panel_parent)->monitor;
+	else
+		return FOOBAR_WIDGET (panel->panel_parent)->monitor;
+}
+
 gboolean
 panel_is_applet_right_stick (GtkWidget *applet)
 {
